Type: Package
Package: DiagrammeR
Title: Graph/Network Visualization
Version: 1.0.12
Authors@R: c(
    person("Richard", "Iannone", , "riannone@me.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3925-190X")),
    person("Olivier", "Roy", , "olivierroy71@hotmail.com", role = "aut")
  )
Description: Build graph/network structures using functions for stepwise
    addition and deletion of nodes and edges. Work with data available in
    tables for bulk addition of nodes, edges, and associated metadata. Use
    graph selections and traversals to apply changes to specific nodes or
    edges. A wide selection of graph algorithms allow for the analysis of
    graphs. Visualize the graphs and take advantage of any aesthetic
    properties assigned to nodes and edges.
License: MIT + file LICENSE
URL: https://rich-iannone.github.io/DiagrammeR/,
        https://github.com/rich-iannone/DiagrammeR
BugReports: https://github.com/rich-iannone/DiagrammeR/issues
Depends: R (>= 4.1)
Imports: dplyr (>= 1.0.7), glue (>= 1.5.0), htmltools (>= 0.5.2),
        htmlwidgets (>= 1.5), igraph (>= 2.0.0), magrittr (>= 1.5),
        purrr (>= 0.3.4), RColorBrewer (>= 1.1-2), readr (>= 2.1.1),
        rlang (>= 1.1.0), cli, rstudioapi (>= 0.7), scales (>= 1.1),
        stringr (>= 1.4), tibble (>= 3.1), tidyr (>= 1.1), viridisLite
        (>= 0.4.2), visNetwork (>= 2.1.0)
Suggests: curl, DiagrammeRsvg, knitr, rmarkdown, rsvg, testthat (>=
        3.1.0), withr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-27 13:46:53 UTC; riannone
Author: Richard Iannone [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3925-190X>),
  Olivier Roy [aut]
Maintainer: Richard Iannone <riannone@me.com>
Repository: CRAN
Date/Publication: 2026-04-27 14:50:02 UTC
