# Generated by roxygen2: do not edit by hand

S3method(print,dgr_graph)
export("%>%")
export(DiagrammeR)
export(DiagrammeROutput)
export(add_balanced_tree)
export(add_cycle)
export(add_edge)
export(add_edge_clone)
export(add_edge_df)
export(add_edges_from_table)
export(add_edges_w_string)
export(add_forward_edges_ws)
export(add_full_graph)
export(add_global_graph_attrs)
export(add_gnm_graph)
export(add_gnp_graph)
export(add_graph_action)
export(add_graph_to_graph_series)
export(add_grid_2d)
export(add_grid_3d)
export(add_growing_graph)
export(add_islands_graph)
export(add_mathjax)
export(add_n_node_clones)
export(add_n_nodes)
export(add_n_nodes_ws)
export(add_node)
export(add_node_clones_ws)
export(add_node_df)
export(add_nodes_from_df_cols)
export(add_nodes_from_table)
export(add_pa_graph)
export(add_path)
export(add_prism)
export(add_reverse_edges_ws)
export(add_smallworld_graph)
export(add_star)
export(clear_selection)
export(colorize_edge_attrs)
export(colorize_node_attrs)
export(combine_edfs)
export(combine_graphs)
export(combine_ndfs)
export(copy_edge_attrs)
export(copy_node_attrs)
export(count_asymmetric_node_pairs)
export(count_automorphisms)
export(count_edges)
export(count_graphs_in_graph_series)
export(count_loop_edges)
export(count_mutual_node_pairs)
export(count_nodes)
export(count_s_connected_cmpts)
export(count_unconnected_node_pairs)
export(count_unconnected_nodes)
export(count_w_connected_cmpts)
export(create_edge_df)
export(create_graph)
export(create_graph_series)
export(create_node_df)
export(delete_cache)
export(delete_edge)
export(delete_edges_ws)
export(delete_global_graph_attrs)
export(delete_graph_actions)
export(delete_loop_edges_ws)
export(delete_node)
export(delete_nodes_ws)
export(deselect_edges)
export(deselect_nodes)
export(display_metagraph)
export(do_bfs)
export(do_dfs)
export(drop_edge_attrs)
export(drop_node_attrs)
export(edge_aes)
export(edge_data)
export(export_csv)
export(export_graph)
export(filter_graph_series)
export(from_adj_matrix)
export(from_igraph)
export(fully_connect_nodes_ws)
export(fully_disconnect_nodes_ws)
export(generate_dot)
export(get_adhesion)
export(get_agg_degree_in)
export(get_agg_degree_out)
export(get_agg_degree_total)
export(get_all_connected_nodes)
export(get_alpha_centrality)
export(get_articulation_points)
export(get_authority_centrality)
export(get_betweenness)
export(get_cache)
export(get_closeness)
export(get_closeness_vitality)
export(get_cmty_edge_btwns)
export(get_cmty_fast_greedy)
export(get_cmty_l_eigenvec)
export(get_cmty_louvain)
export(get_cmty_walktrap)
export(get_common_nbrs)
export(get_coreness)
export(get_degree_distribution)
export(get_degree_histogram)
export(get_degree_in)
export(get_degree_out)
export(get_degree_total)
export(get_dice_similarity)
export(get_eccentricity)
export(get_edge_attrs)
export(get_edge_attrs_ws)
export(get_edge_count_w_multiedge)
export(get_edge_df)
export(get_edge_df_ws)
export(get_edge_ids)
export(get_edge_info)
export(get_edges)
export(get_eigen_centrality)
export(get_girth)
export(get_global_graph_attr_info)
export(get_graph_actions)
export(get_graph_from_graph_series)
export(get_graph_info)
export(get_graph_log)
export(get_graph_name)
export(get_graph_series_info)
export(get_graph_time)
export(get_jaccard_similarity)
export(get_last_edges_created)
export(get_last_nodes_created)
export(get_leverage_centrality)
export(get_max_eccentricity)
export(get_mean_distance)
export(get_min_cut_between)
export(get_min_eccentricity)
export(get_multiedge_count)
export(get_nbrs)
export(get_node_attrs)
export(get_node_attrs_ws)
export(get_node_df)
export(get_node_df_ws)
export(get_node_ids)
export(get_node_info)
export(get_non_nbrs)
export(get_pagerank)
export(get_paths)
export(get_periphery)
export(get_predecessors)
export(get_radiality)
export(get_reciprocity)
export(get_s_connected_cmpts)
export(get_selection)
export(get_similar_nbrs)
export(get_successors)
export(get_w_connected_cmpts)
export(grViz)
export(grVizOutput)
export(import_graph)
export(invert_selection)
export(is_edge_loop)
export(is_edge_multiple)
export(is_edge_mutual)
export(is_edge_present)
export(is_graph_connected)
export(is_graph_dag)
export(is_graph_directed)
export(is_graph_empty)
export(is_graph_simple)
export(is_graph_undirected)
export(is_graph_weighted)
export(is_node_present)
export(is_property_graph)
export(join_edge_attrs)
export(join_node_attrs)
export(layout_nodes_w_string)
export(mermaid)
export(mutate_edge_attrs)
export(mutate_edge_attrs_ws)
export(mutate_node_attrs)
export(mutate_node_attrs_ws)
export(node_aes)
export(node_data)
export(nudge_node_positions_ws)
export(open_graph)
export(recode_edge_attrs)
export(recode_node_attrs)
export(remove_graph_from_graph_series)
export(rename_edge_attrs)
export(rename_node_attrs)
export(renderDiagrammeR)
export(renderGrViz)
export(render_graph)
export(render_graph_from_graph_series)
export(reorder_graph_actions)
export(replace_in_spec)
export(rescale_edge_attrs)
export(rescale_node_attrs)
export(rev_edge_dir)
export(rev_edge_dir_ws)
export(save_graph)
export(select_edges)
export(select_edges_by_edge_id)
export(select_edges_by_node_id)
export(select_last_edges_created)
export(select_last_nodes_created)
export(select_nodes)
export(select_nodes_by_degree)
export(select_nodes_by_id)
export(select_nodes_in_neighborhood)
export(set_cache)
export(set_edge_attr_to_display)
export(set_edge_attrs)
export(set_edge_attrs_ws)
export(set_graph_directed)
export(set_graph_name)
export(set_graph_time)
export(set_graph_undirected)
export(set_node_attr_to_display)
export(set_node_attr_w_fcn)
export(set_node_attrs)
export(set_node_attrs_ws)
export(set_node_position)
export(to_igraph)
export(transform_to_complement_graph)
export(transform_to_min_spanning_tree)
export(transform_to_subgraph_ws)
export(trav_both)
export(trav_both_edge)
export(trav_in)
export(trav_in_edge)
export(trav_in_node)
export(trav_in_until)
export(trav_out)
export(trav_out_edge)
export(trav_out_node)
export(trav_out_until)
export(trav_reverse_edge)
export(trigger_graph_actions)
export(visnetwork)
export(x11_hex)
import(RColorBrewer)
import(glue)
import(htmlwidgets)
import(rlang)
import(visNetwork)
importFrom(magrittr,"%>%")
importFrom(magrittr,not)
