#nocov start

utils::globalVariables(
  c(
    ".",
    "action_index",
    "action_name",
    "attr",
    "attr_type",
    "deg",
    "degree",
    "df_id",
    "df_id__",
    "display",
    "duration",
    "edge",
    "edges",
    "edge_from_to",
    "e_id",
    "fa_uri",
    "fillcolor",
    "from",
    "from.y",
    "from_label",
    "from_to",
    "from_type",
    "function_used",
    "hex",
    "id",
    "id_external",
    "id.x",
    "id.y",
    "id__",
    "indeg",
    "indegree",
    "indegree_dist",
    "indegree_hist",
    "index",
    "index__",
    "image",
    "label",
    "label_col",
    "n",
    "new_fillcolor",
    "new_node_id",
    "node",
    "nodes",
    "node_edge__",
    "node_id",
    "outdeg",
    "outdegree",
    "outdegree_dist",
    "outdegree_hist",
    "rel",
    "step_created_edges",
    "step_created_nodes",
    "step_deleted_edges",
    "step_deleted_nodes",
    "step_init_with_edges",
    "step_init_with_nodes",
    "string",
    "time_modified",
    "to",
    "to.y",
    "to_label",
    "to_type",
    "total_degree",
    "total_degree_dist",
    "total_degree_hist",
    "type",
    "V1",
    "V2",
    "value",
    "value.x",
    "value_x",
    "value.y",
    "version_id",
    "x",
    "y"
    )
  )

#nocov end
