#' Options for node and edge aesthetics and data
#'
#' @param node_aes An optional list of named vectors comprising node aesthetic
#'   attributes. The helper function [node_aes()] is strongly recommended for
#'   use here as it contains arguments for each of the accepted node aesthetic
#'   attributes (e.g., `shape`, `style`, `color`, `fillcolor`).
#' @param edge_aes An optional list of named vectors comprising edge aesthetic
#'   attributes. The helper function [edge_aes()] is strongly recommended for
#'   use here as it contains arguments for each of the accepted edge aesthetic
#'   attributes (e.g., `shape`, `style`, `penwidth`, `color`).
#' @param node_data An optional list of named vectors comprising node data
#'   attributes. The helper function [node_data()] is strongly recommended for
#'   use here as it helps bind data specifically to the created nodes.
#' @param edge_data An optional list of named vectors comprising edge data
#'   attributes. The helper function [edge_data()] is strongly recommended for
#'   use here as it helps bind data specifically to the created edges.
#' @name node_edge_aes_data
#'
#' @keywords internal
#' @family aesthetics
NULL
