# Using an invalid graph triggers an error.

    Code
      add_balanced_tree(invalid_graph)
    Condition
      Error in `add_balanced_tree()`:
      ! The graph is not valid.
    Code
      add_cycle(invalid_graph)
    Condition
      Error in `add_cycle()`:
      ! The graph is not valid.
    Code
      add_edge(invalid_graph)
    Condition
      Error in `add_edge()`:
      ! The graph is not valid.
    Code
      add_edge_clone(invalid_graph)
    Condition
      Error in `add_edge_clone()`:
      ! The graph is not valid.
    Code
      add_edge_df(invalid_graph)
    Condition
      Error in `add_edge_df()`:
      ! The graph is not valid.
    Code
      add_edges_from_table(invalid_graph)
    Condition
      Error in `add_edges_from_table()`:
      ! The graph is not valid.
    Code
      add_edges_w_string(invalid_graph)
    Condition
      Error in `add_edges_w_string()`:
      ! The graph is not valid.
    Code
      add_forward_edges_ws(invalid_graph)
    Condition
      Error in `add_forward_edges_ws()`:
      ! The graph is not valid.
    Code
      add_full_graph(invalid_graph)
    Condition
      Error in `add_full_graph()`:
      ! The graph is not valid.
    Code
      add_global_graph_attrs(invalid_graph)
    Condition
      Error in `add_global_graph_attrs()`:
      ! The graph is not valid.
    Code
      add_gnm_graph(invalid_graph)
    Condition
      Error in `add_gnm_graph()`:
      ! The graph is not valid.
    Code
      add_gnp_graph(invalid_graph)
    Condition
      Error in `add_gnp_graph()`:
      ! The graph is not valid.
    Code
      add_graph_action(invalid_graph)
    Condition
      Error in `add_graph_action()`:
      ! The graph is not valid.
    Code
      add_graph_to_graph_series(graph_series = invalid_graph)
    Condition
      Error in `add_graph_to_graph_series()`:
      ! `graph` is absent but must be supplied.
    Code
      add_grid_2d(invalid_graph)
    Condition
      Error in `add_grid_2d()`:
      ! The graph is not valid.
    Code
      add_grid_3d(invalid_graph)
    Condition
      Error in `add_grid_3d()`:
      ! The graph is not valid.
    Code
      add_growing_graph(invalid_graph)
    Condition
      Error in `add_growing_graph()`:
      ! The graph is not valid.
    Code
      add_islands_graph(invalid_graph)
    Condition
      Error in `add_islands_graph()`:
      ! The graph is not valid.
    Code
      add_n_node_clones(invalid_graph)
    Condition
      Error in `add_n_node_clones()`:
      ! The graph is not valid.
    Code
      add_n_nodes(invalid_graph)
    Condition
      Error in `add_n_nodes()`:
      ! The graph is not valid.
    Code
      add_n_nodes_ws(invalid_graph)
    Condition
      Error in `add_n_nodes_ws()`:
      ! The graph is not valid.
    Code
      add_node(invalid_graph)
    Condition
      Error in `add_node()`:
      ! The graph is not valid.
    Code
      add_node_clones_ws(invalid_graph)
    Condition
      Error in `add_node_clones_ws()`:
      ! The graph is not valid.
    Code
      add_node_df(invalid_graph)
    Condition
      Error in `add_node_df()`:
      ! The graph is not valid.
    Code
      add_nodes_from_df_cols(invalid_graph)
    Condition
      Error in `add_nodes_from_df_cols()`:
      ! The graph is not valid.
    Code
      add_nodes_from_table(invalid_graph)
    Condition
      Error in `add_nodes_from_table()`:
      ! The graph is not valid.
    Code
      add_pa_graph(invalid_graph)
    Condition
      Error in `add_pa_graph()`:
      ! The graph is not valid.
    Code
      add_path(invalid_graph)
    Condition
      Error in `add_path()`:
      ! The graph is not valid.
    Code
      add_prism(invalid_graph)
    Condition
      Error in `add_prism()`:
      ! The graph is not valid.
    Code
      add_smallworld_graph(invalid_graph)
    Condition
      Error in `add_smallworld_graph()`:
      ! The graph is not valid.
    Code
      add_star(invalid_graph)
    Condition
      Error in `add_star()`:
      ! The graph is not valid.
    Code
      add_reverse_edges_ws(invalid_graph)
    Condition
      Error in `add_reverse_edges_ws()`:
      ! The graph is not valid.
    Code
      clear_selection(invalid_graph)
    Condition
      Error in `clear_selection()`:
      ! The graph is not valid.
    Code
      colorize_edge_attrs(invalid_graph)
    Condition
      Error in `colorize_edge_attrs()`:
      ! The graph is not valid.
    Code
      colorize_node_attrs(invalid_graph)
    Condition
      Error in `colorize_node_attrs()`:
      ! The graph is not valid.
    Code
      copy_edge_attrs(invalid_graph)
    Condition
      Error in `copy_edge_attrs()`:
      ! The graph is not valid.
    Code
      copy_node_attrs(invalid_graph)
    Condition
      Error in `copy_node_attrs()`:
      ! The graph is not valid.
    Code
      count_asymmetric_node_pairs(invalid_graph)
    Condition
      Error in `count_asymmetric_node_pairs()`:
      ! The graph is not valid.
    Code
      count_automorphisms(invalid_graph)
    Condition
      Error in `count_automorphisms()`:
      ! The graph is not valid.
    Code
      count_edges(invalid_graph)
    Condition
      Error in `count_edges()`:
      ! The graph is not valid.
    Code
      count_loop_edges(invalid_graph)
    Condition
      Error in `count_loop_edges()`:
      ! The graph is not valid.
    Code
      count_mutual_node_pairs(invalid_graph)
    Condition
      Error in `count_mutual_node_pairs()`:
      ! The graph is not valid.
    Code
      count_nodes(invalid_graph)
    Condition
      Error in `count_nodes()`:
      ! The graph is not valid.
    Code
      count_s_connected_cmpts(invalid_graph)
    Condition
      Error in `count_s_connected_cmpts()`:
      ! The graph is not valid.
    Code
      count_unconnected_node_pairs(invalid_graph)
    Condition
      Error in `count_unconnected_node_pairs()`:
      ! The graph is not valid.
    Code
      count_unconnected_nodes(invalid_graph)
    Condition
      Error in `count_unconnected_nodes()`:
      ! The graph is not valid.
    Code
      count_w_connected_cmpts(invalid_graph)
    Condition
      Error in `count_w_connected_cmpts()`:
      ! The graph is not valid.
    Code
      create_graph_series(invalid_graph)
    Condition
      Error in `create_graph_series()`:
      ! The graph is not valid.
    Code
      delete_edge(invalid_graph)
    Condition
      Error in `delete_edge()`:
      ! The graph is not valid.
    Code
      delete_edges_ws(invalid_graph)
    Condition
      Error in `delete_edges_ws()`:
      ! The graph is not valid.
    Code
      delete_global_graph_attrs(invalid_graph)
    Condition
      Error in `delete_global_graph_attrs()`:
      ! The graph is not valid.
    Code
      delete_graph_actions(invalid_graph)
    Condition
      Error in `delete_graph_actions()`:
      ! The graph is not valid.
    Code
      delete_loop_edges_ws(invalid_graph)
    Condition
      Error in `delete_loop_edges_ws()`:
      ! The graph is not valid.
    Code
      deselect_edges(invalid_graph)
    Condition
      Error in `deselect_edges()`:
      ! The graph is not valid.
    Code
      deselect_nodes(invalid_graph)
    Condition
      Error in `deselect_nodes()`:
      ! The graph is not valid.
    Code
      delete_node(invalid_graph)
    Condition
      Error in `delete_node()`:
      ! The graph is not valid.
    Code
      delete_nodes_ws(invalid_graph)
    Condition
      Error in `delete_nodes_ws()`:
      ! The graph is not valid.
    Code
      display_metagraph(invalid_graph)
    Condition
      Error in `display_metagraph()`:
      ! The graph is not valid.
    Code
      do_bfs(invalid_graph)
    Condition
      Error in `do_bfs()`:
      ! The graph is not valid.
    Code
      do_dfs(invalid_graph)
    Condition
      Error in `do_dfs()`:
      ! The graph is not valid.
    Code
      drop_edge_attrs(invalid_graph)
    Condition
      Error in `drop_edge_attrs()`:
      ! The graph is not valid.
    Code
      drop_node_attrs(invalid_graph)
    Condition
      Error in `drop_node_attrs()`:
      ! The graph is not valid.
    Code
      export_csv(invalid_graph)
    Condition
      Error in `export_csv()`:
      ! The graph is not valid.
    Code
      export_graph(invalid_graph)
    Condition
      Error in `export_graph()`:
      ! The graph is not valid.
    Code
      fully_connect_nodes_ws(invalid_graph)
    Condition
      Error in `fully_connect_nodes_ws()`:
      ! The graph is not valid.
    Code
      fully_disconnect_nodes_ws(invalid_graph)
    Condition
      Error in `fully_disconnect_nodes_ws()`:
      ! The graph is not valid.
    Code
      generate_dot(invalid_graph)
    Condition
      Error in `generate_dot()`:
      ! The graph is not valid.
    Code
      get_adhesion(invalid_graph)
    Condition
      Error in `get_adhesion()`:
      ! The graph is not valid.
    Code
      get_agg_degree_in(invalid_graph)
    Condition
      Error in `get_agg_degree_in()`:
      ! The graph is not valid.
    Code
      get_agg_degree_out(invalid_graph)
    Condition
      Error in `get_agg_degree_out()`:
      ! The graph is not valid.
    Code
      get_agg_degree_total(invalid_graph)
    Condition
      Error in `get_agg_degree_total()`:
      ! The graph is not valid.
    Code
      get_all_connected_nodes(invalid_graph)
    Condition
      Error in `get_all_connected_nodes()`:
      ! The graph is not valid.
    Code
      get_alpha_centrality(invalid_graph)
    Condition
      Error in `get_alpha_centrality()`:
      ! The graph is not valid.
    Code
      get_articulation_points(invalid_graph)
    Condition
      Error in `get_articulation_points()`:
      ! The graph is not valid.
    Code
      get_authority_centrality(invalid_graph)
    Condition
      Error in `get_authority_centrality()`:
      ! The graph is not valid.
    Code
      get_betweenness(invalid_graph)
    Condition
      Error in `get_betweenness()`:
      ! The graph is not valid.
    Code
      get_cache(invalid_graph)
    Condition
      Error in `get_cache()`:
      ! The graph is not valid.
    Code
      get_closeness(invalid_graph)
    Condition
      Error in `get_closeness()`:
      ! The graph is not valid.
    Code
      get_closeness_vitality(invalid_graph)
    Condition
      Error in `get_closeness_vitality()`:
      ! The graph is not valid.
    Code
      get_cmty_edge_btwns(invalid_graph)
    Condition
      Error in `get_cmty_edge_btwns()`:
      ! The graph is not valid.
    Code
      get_cmty_fast_greedy(invalid_graph)
    Condition
      Error in `get_cmty_fast_greedy()`:
      ! The graph is not valid.
    Code
      get_cmty_l_eigenvec(invalid_graph)
    Condition
      Error in `get_cmty_l_eigenvec()`:
      ! The graph is not valid.
    Code
      get_cmty_louvain(invalid_graph)
    Condition
      Error in `get_cmty_louvain()`:
      ! The graph is not valid.
    Code
      get_cmty_walktrap(invalid_graph)
    Condition
      Error in `get_cmty_walktrap()`:
      ! The graph is not valid.
    Code
      get_common_nbrs(invalid_graph)
    Condition
      Error in `get_common_nbrs()`:
      ! The graph is not valid.
    Code
      get_coreness(invalid_graph)
    Condition
      Error in `get_coreness()`:
      ! The graph is not valid.
    Code
      get_degree_distribution(invalid_graph)
    Condition
      Error in `get_degree_distribution()`:
      ! The graph is not valid.
    Code
      get_degree_histogram(invalid_graph)
    Condition
      Error in `get_degree_histogram()`:
      ! The graph is not valid.
    Code
      get_degree_in(invalid_graph)
    Condition
      Error in `get_degree_in()`:
      ! The graph is not valid.
    Code
      get_degree_out(invalid_graph)
    Condition
      Error in `get_degree_out()`:
      ! The graph is not valid.
    Code
      get_degree_total(invalid_graph)
    Condition
      Error in `get_degree_total()`:
      ! The graph is not valid.
    Code
      get_dice_similarity(invalid_graph)
    Condition
      Error in `get_dice_similarity()`:
      ! The graph is not valid.
    Code
      get_eccentricity(invalid_graph)
    Condition
      Error in `get_eccentricity()`:
      ! The graph is not valid.
    Code
      get_edge_attrs(invalid_graph)
    Condition
      Error in `get_edge_attrs()`:
      ! The graph is not valid.
    Code
      get_edge_attrs_ws(invalid_graph)
    Condition
      Error in `get_edge_attrs_ws()`:
      ! The graph is not valid.
    Code
      get_edge_count_w_multiedge(invalid_graph)
    Condition
      Error in `get_edge_count_w_multiedge()`:
      ! The graph is not valid.
    Code
      get_edge_df(invalid_graph)
    Condition
      Error in `get_edge_df()`:
      ! The graph is not valid.
    Code
      get_edge_df_ws(invalid_graph)
    Condition
      Error in `get_edge_df_ws()`:
      ! The graph is not valid.
    Code
      get_edge_info(invalid_graph)
    Condition
      Error in `get_edge_info()`:
      ! The graph is not valid.
    Code
      get_eigen_centrality(invalid_graph)
    Condition
      Error in `get_eigen_centrality()`:
      ! The graph is not valid.
    Code
      get_girth(invalid_graph)
    Condition
      Error in `get_girth()`:
      ! The graph is not valid.
    Code
      get_global_graph_attr_info(invalid_graph)
    Condition
      Error in `get_global_graph_attr_info()`:
      ! The graph is not valid.
    Code
      get_graph_actions(invalid_graph)
    Condition
      Error in `get_graph_actions()`:
      ! The graph is not valid.
    Code
      get_graph_log(invalid_graph)
    Condition
      Error in `get_graph_log()`:
      ! The graph is not valid.
    Code
      get_graph_name(invalid_graph)
    Condition
      Error in `get_graph_name()`:
      ! The graph is not valid.
    Code
      get_graph_time(invalid_graph)
    Condition
      Error in `get_graph_time()`:
      ! The graph is not valid.
    Code
      get_jaccard_similarity(invalid_graph)
    Condition
      Error in `get_jaccard_similarity()`:
      ! The graph is not valid.
    Code
      get_last_edges_created(invalid_graph)
    Condition
      Error in `get_last_edges_created()`:
      ! The graph is not valid.
    Code
      get_last_nodes_created(invalid_graph)
    Condition
      Error in `get_last_nodes_created()`:
      ! The graph is not valid.
    Code
      get_leverage_centrality(invalid_graph)
    Condition
      Error in `get_leverage_centrality()`:
      ! The graph is not valid.
    Code
      get_max_eccentricity(invalid_graph)
    Condition
      Error in `get_max_eccentricity()`:
      ! The graph is not valid.
    Code
      get_mean_distance(invalid_graph)
    Condition
      Error in `get_mean_distance()`:
      ! The graph is not valid.
    Code
      get_min_cut_between(invalid_graph)
    Condition
      Error in `get_min_cut_between()`:
      ! The graph is not valid.
    Code
      get_min_eccentricity(invalid_graph)
    Condition
      Error in `get_min_eccentricity()`:
      ! The graph is not valid.
    Code
      get_multiedge_count(invalid_graph)
    Condition
      Error in `get_multiedge_count()`:
      ! The graph is not valid.
    Code
      get_nbrs(invalid_graph)
    Condition
      Error in `get_nbrs()`:
      ! The graph is not valid.
    Code
      get_node_attrs(invalid_graph)
    Condition
      Error in `get_node_attrs()`:
      ! The graph is not valid.
    Code
      get_node_attrs_ws(invalid_graph)
    Condition
      Error in `get_node_attrs_ws()`:
      ! The graph is not valid.
    Code
      get_node_df(invalid_graph)
    Condition
      Error in `get_node_df()`:
      ! The graph is not valid.
    Code
      get_node_df_ws(invalid_graph)
    Condition
      Error in `get_node_df_ws()`:
      ! The graph is not valid.
    Code
      get_node_ids(invalid_graph)
    Condition
      Error in `is_graph_empty()`:
      ! The graph is not valid.
    Code
      get_node_info(invalid_graph)
    Condition
      Error in `get_node_info()`:
      ! The graph is not valid.
    Code
      get_non_nbrs(invalid_graph)
    Condition
      Error in `get_non_nbrs()`:
      ! The graph is not valid.
    Code
      get_pagerank(invalid_graph)
    Condition
      Error in `get_pagerank()`:
      ! The graph is not valid.
    Code
      get_paths(invalid_graph)
    Condition
      Error in `get_paths()`:
      ! The graph is not valid.
    Code
      get_periphery(invalid_graph)
    Condition
      Error in `get_periphery()`:
      ! The graph is not valid.
    Code
      get_predecessors(invalid_graph)
    Condition
      Error in `get_predecessors()`:
      ! The graph is not valid.
    Code
      get_radiality(invalid_graph)
    Condition
      Error in `get_radiality()`:
      ! The graph is not valid.
    Code
      get_reciprocity(invalid_graph)
    Condition
      Error in `get_reciprocity()`:
      ! The graph is not valid.
    Code
      get_s_connected_cmpts(invalid_graph)
    Condition
      Error in `get_s_connected_cmpts()`:
      ! The graph is not valid.
    Code
      get_selection(invalid_graph)
    Condition
      Error in `get_selection()`:
      ! The graph is not valid.
    Code
      get_similar_nbrs(invalid_graph)
    Condition
      Error in `get_similar_nbrs()`:
      ! The graph is not valid.
    Code
      get_successors(invalid_graph)
    Condition
      Error in `get_successors()`:
      ! The graph is not valid.
    Code
      get_w_connected_cmpts(invalid_graph)
    Condition
      Error in `get_w_connected_cmpts()`:
      ! The graph is not valid.
    Code
      get_graph_info(invalid_graph)
    Condition
      Error in `get_graph_info()`:
      ! The graph is not valid.
    Code
      invert_selection(invalid_graph)
    Condition
      Error in `invert_selection()`:
      ! The graph is not valid.
    Code
      is_edge_loop(invalid_graph)
    Condition
      Error in `is_edge_loop()`:
      ! The graph is not valid.
    Code
      is_edge_multiple(invalid_graph)
    Condition
      Error in `is_edge_multiple()`:
      ! The graph is not valid.
    Code
      is_edge_mutual(invalid_graph)
    Condition
      Error in `is_edge_mutual()`:
      ! The graph is not valid.
    Code
      is_edge_present(invalid_graph)
    Condition
      Error in `is_edge_present()`:
      ! The graph is not valid.
    Code
      is_graph_connected(invalid_graph)
    Condition
      Error in `is_graph_connected()`:
      ! The graph is not valid.
    Code
      is_graph_dag(invalid_graph)
    Condition
      Error in `is_graph_dag()`:
      ! The graph is not valid.
    Code
      is_graph_directed(invalid_graph)
    Condition
      Error in `is_graph_directed()`:
      ! The graph is not valid.
    Code
      is_graph_empty(invalid_graph)
    Condition
      Error in `is_graph_empty()`:
      ! The graph is not valid.
    Code
      is_graph_simple(invalid_graph)
    Condition
      Error in `is_graph_simple()`:
      ! The graph is not valid.
    Code
      is_graph_undirected(invalid_graph)
    Condition
      Error in `is_graph_undirected()`:
      ! The graph is not valid.
    Code
      is_graph_weighted(invalid_graph)
    Condition
      Error in `is_graph_weighted()`:
      ! The graph is not valid.
    Code
      is_node_present(invalid_graph)
    Condition
      Error in `is_node_present()`:
      ! The graph is not valid.
    Code
      is_property_graph(invalid_graph)
    Condition
      Error in `is_property_graph()`:
      ! The graph is not valid.
    Code
      join_edge_attrs(invalid_graph)
    Condition
      Error in `join_edge_attrs()`:
      ! The graph is not valid.
    Code
      join_node_attrs(invalid_graph)
    Condition
      Error in `join_node_attrs()`:
      ! The graph is not valid.
    Code
      layout_nodes_w_string(invalid_graph)
    Condition
      Error in `layout_nodes_w_string()`:
      ! The graph is not valid.
    Code
      mutate_edge_attrs(invalid_graph)
    Condition
      Error in `mutate_edge_attrs()`:
      ! The graph is not valid.
    Code
      mutate_edge_attrs_ws(invalid_graph)
    Condition
      Error in `mutate_edge_attrs_ws()`:
      ! The graph is not valid.
    Code
      mutate_node_attrs(invalid_graph)
    Condition
      Error in `mutate_node_attrs()`:
      ! The graph is not valid.
    Code
      mutate_node_attrs_ws(invalid_graph)
    Condition
      Error in `mutate_node_attrs_ws()`:
      ! The graph is not valid.
    Code
      nudge_node_positions_ws(invalid_graph)
    Condition
      Error in `nudge_node_positions_ws()`:
      ! The graph is not valid.
    Code
      recode_edge_attrs(invalid_graph)
    Condition
      Error in `recode_edge_attrs()`:
      ! The graph is not valid.
    Code
      recode_node_attrs(invalid_graph)
    Condition
      Error in `recode_node_attrs()`:
      ! The graph is not valid.
    Code
      rename_edge_attrs(invalid_graph)
    Condition
      Error in `rename_edge_attrs()`:
      ! The graph is not valid.
    Code
      rename_node_attrs(invalid_graph)
    Condition
      Error in `rename_node_attrs()`:
      ! The graph is not valid.
    Code
      render_graph(invalid_graph)
    Condition
      Error in `render_graph()`:
      ! The graph is not valid.
    Code
      reorder_graph_actions(invalid_graph)
    Condition
      Error in `reorder_graph_actions()`:
      ! The graph is not valid.
    Code
      rescale_edge_attrs(invalid_graph)
    Condition
      Error in `rescale_edge_attrs()`:
      ! The graph is not valid.
    Code
      rescale_node_attrs(invalid_graph)
    Condition
      Error in `rescale_node_attrs()`:
      ! The graph is not valid.
    Code
      rev_edge_dir(invalid_graph)
    Condition
      Error in `rev_edge_dir()`:
      ! The graph is not valid.
    Code
      rev_edge_dir_ws(invalid_graph)
    Condition
      Error in `rev_edge_dir_ws()`:
      ! The graph is not valid.
    Code
      select_edges(invalid_graph)
    Condition
      Error in `select_edges()`:
      ! The graph is not valid.
    Code
      select_edges_by_edge_id(invalid_graph)
    Condition
      Error in `select_edges_by_edge_id()`:
      ! The graph is not valid.
    Code
      select_edges_by_node_id(invalid_graph)
    Condition
      Error in `select_edges_by_node_id()`:
      ! The graph is not valid.
    Code
      select_last_edges_created(invalid_graph)
    Condition
      Error in `select_last_edges_created()`:
      ! The graph is not valid.
    Code
      select_last_nodes_created(invalid_graph)
    Condition
      Error in `select_last_nodes_created()`:
      ! The graph is not valid.
    Code
      select_nodes(invalid_graph)
    Condition
      Error in `select_nodes()`:
      ! The graph is not valid.
    Code
      select_nodes_by_degree(invalid_graph)
    Condition
      Error in `select_nodes_by_degree()`:
      ! The graph is not valid.
    Code
      select_nodes_by_id(invalid_graph)
    Condition
      Error in `select_nodes_by_id()`:
      ! The graph is not valid.
    Code
      select_nodes_in_neighborhood(invalid_graph)
    Condition
      Error in `select_nodes_in_neighborhood()`:
      ! The graph is not valid.
    Code
      set_cache(invalid_graph)
    Condition
      Error in `set_cache()`:
      ! The graph is not valid.
    Code
      set_edge_attr_to_display(invalid_graph)
    Condition
      Error in `set_edge_attr_to_display()`:
      ! The graph is not valid.
    Code
      set_edge_attrs(invalid_graph)
    Condition
      Error in `set_edge_attrs()`:
      ! argument "values" is missing, with no default
    Code
      set_edge_attrs_ws(invalid_graph)
    Condition
      Error in `set_edge_attrs_ws()`:
      ! The graph is not valid.
    Code
      set_graph_directed(invalid_graph)
    Condition
      Error in `set_graph_directed()`:
      ! The graph is not valid.
    Code
      set_graph_name(invalid_graph)
    Condition
      Error in `set_graph_name()`:
      ! The graph is not valid.
    Code
      set_graph_time(invalid_graph)
    Condition
      Error in `set_graph_time()`:
      ! The graph is not valid.
    Code
      set_graph_undirected(invalid_graph)
    Condition
      Error in `set_graph_undirected()`:
      ! The graph is not valid.
    Code
      set_node_attr_to_display(invalid_graph)
    Condition
      Error in `set_node_attr_to_display()`:
      ! The graph is not valid.
    Code
      set_node_attr_w_fcn(invalid_graph)
    Condition
      Error in `set_node_attr_w_fcn()`:
      ! The graph is not valid.
    Code
      set_node_attrs(invalid_graph)
    Condition
      Error in `set_node_attrs()`:
      ! argument "values" is missing, with no default
    Code
      set_node_attrs_ws(invalid_graph)
    Condition
      Error in `set_node_attrs_ws()`:
      ! The graph is not valid.
    Code
      set_node_position(invalid_graph)
    Condition
      Error in `set_node_position()`:
      ! The graph is not valid.
    Code
      to_igraph(invalid_graph)
    Condition
      Error in `to_igraph()`:
      ! The graph is not valid.
    Code
      transform_to_complement_graph(invalid_graph)
    Condition
      Error in `transform_to_complement_graph()`:
      ! The graph is not valid.
    Code
      transform_to_min_spanning_tree(invalid_graph)
    Condition
      Error in `transform_to_min_spanning_tree()`:
      ! The graph is not valid.
    Code
      transform_to_subgraph_ws(invalid_graph)
    Condition
      Error in `transform_to_subgraph_ws()`:
      ! The graph is not valid.
    Code
      trav_both(invalid_graph)
    Condition
      Error in `trav_both()`:
      ! The graph is not valid.
    Code
      trav_both_edge(invalid_graph)
    Condition
      Error in `trav_both_edge()`:
      ! The graph is not valid.
    Code
      trav_in(invalid_graph)
    Condition
      Error in `trav_in()`:
      ! The graph is not valid.
    Code
      trav_in_edge(invalid_graph)
    Condition
      Error in `trav_in_edge()`:
      ! The graph is not valid.
    Code
      trav_in_node(invalid_graph)
    Condition
      Error in `trav_in_node()`:
      ! The graph is not valid.
    Code
      trav_in_until(invalid_graph)
    Condition
      Error in `trav_in_until()`:
      ! The graph is not valid.
    Code
      trav_reverse_edge(invalid_graph)
    Condition
      Error in `trav_reverse_edge()`:
      ! The graph is not valid.
    Code
      trav_out(invalid_graph)
    Condition
      Error in `trav_out()`:
      ! The graph is not valid.
    Code
      trav_out_edge(invalid_graph)
    Condition
      Error in `trav_out_edge()`:
      ! The graph is not valid.
    Code
      trav_out_node(invalid_graph)
    Condition
      Error in `trav_out_node()`:
      ! The graph is not valid.
    Code
      trav_out_until(invalid_graph)
    Condition
      Error in `trav_out_until()`:
      ! The graph is not valid.
    Code
      trav_reverse_edge(invalid_graph)
    Condition
      Error in `trav_reverse_edge()`:
      ! The graph is not valid.
    Code
      trigger_graph_actions(invalid_graph)
    Condition
      Error in `trigger_graph_actions()`:
      ! The graph is not valid.

# Using an empty graph with trigger an error

    Code
      add_edge(empty_graph)
    Condition
      Error in `add_edge()`:
      ! The graph does not contain nodes.
      * An edge cannot be added.
    Code
      add_edge_df(empty_graph)
    Condition
      Error in `add_edge_df()`:
      ! The graph does not contain nodes.
      * So, edges cannot be added.

# Using a graph with no edges with trigger an error

    Code
      add_edge_clone(graph_no_edges)
    Condition
      Error in `add_edge_clone()`:
      ! The graph contains no edges.
    Code
      add_forward_edges_ws(graph_no_edges)
    Condition
      Error in `add_forward_edges_ws()`:
      ! The graph contains no edges.
    Code
      add_reverse_edges_ws(graph_no_edges)
    Condition
      Error in `add_reverse_edges_ws()`:
      ! The graph contains no edges.
    Code
      delete_edge(graph_no_edges)
    Condition
      Error in `delete_edge()`:
      ! The graph contains no edges.
    Code
      delete_edges_ws(graph_no_edges)
    Condition
      Error in `delete_edges_ws()`:
      ! The graph contains no edges.
    Code
      get_edge_count_w_multiedge(graph_no_edges)
    Condition
      Error in `get_edge_count_w_multiedge()`:
      ! The graph contains no edges.
    Code
      get_last_edges_created(graph_no_edges)
    Condition
      Error in `get_last_edges_created()`:
      ! The graph contains no edges.
    Code
      get_multiedge_count(graph_no_edges)
    Condition
      Error in `get_multiedge_count()`:
      ! The graph contains no edges.
    Code
      is_edge_loop(graph_no_edges)
    Condition
      Error in `is_edge_loop()`:
      ! The graph contains no edges.
    Code
      is_edge_multiple(graph_no_edges)
    Condition
      Error in `is_edge_multiple()`:
      ! The graph contains no edges.
    Code
      is_edge_mutual(graph_no_edges)
    Condition
      Error in `is_edge_mutual()`:
      ! The graph contains no edges.
    Code
      mutate_edge_attrs_ws(graph_no_edges)
    Condition
      Error in `mutate_edge_attrs_ws()`:
      ! The graph contains no edges.
    Code
      recode_edge_attrs(graph_no_edges)
    Condition
      Error in `recode_edge_attrs()`:
      ! The graph contains no edges.
    Code
      rename_edge_attrs(graph_no_edges)
    Condition
      Error in `rename_edge_attrs()`:
      ! The graph contains no edges.
    Code
      rescale_edge_attrs(graph_no_edges)
    Condition
      Error in `rescale_edge_attrs()`:
      ! The graph contains no edges.
    Code
      rev_edge_dir(graph_no_edges)
    Condition
      Error in `rev_edge_dir()`:
      ! The graph contains no edges.
    Code
      rev_edge_dir_ws(graph_no_edges)
    Condition
      Error in `rev_edge_dir_ws()`:
      ! The graph contains no edges.
    Code
      select_edges(graph_no_edges)
    Condition
      Error in `select_edges()`:
      ! The graph contains no edges.
    Code
      select_edges_by_edge_id(graph_no_edges)
    Condition
      Error in `select_edges_by_edge_id()`:
      ! The graph contains no edges.
    Code
      select_edges_by_node_id(graph_no_edges)
    Condition
      Error in `select_edges_by_node_id()`:
      ! The graph contains no edges.
    Code
      select_last_edges_created(graph_no_edges)
    Condition
      Error in `select_last_edges_created()`:
      ! The graph contains no edges.
    Code
      set_edge_attr_to_display(graph_no_edges)
    Condition
      Error in `set_edge_attr_to_display()`:
      ! The graph contains no edges.
    Code
      set_edge_attrs_ws(graph_no_edges)
    Condition
      Error in `set_edge_attrs_ws()`:
      ! The graph contains no edges.
    Code
      trav_both(graph_no_edges)
    Condition
      Error in `trav_both()`:
      ! The graph contains no edges.
    Code
      trav_both_edge(graph_no_edges)
    Condition
      Error in `trav_both_edge()`:
      ! The graph contains no edges.
    Code
      trav_in(graph_no_edges)
    Condition
      Error in `trav_in()`:
      ! The graph contains no edges.
    Code
      trav_in_edge(graph_no_edges)
    Condition
      Error in `trav_in_edge()`:
      ! The graph contains no edges.
    Code
      trav_in_node(graph_no_edges)
    Condition
      Error in `trav_in_node()`:
      ! The graph contains no edges.
    Code
      trav_in_until(graph_no_edges)
    Condition
      Error in `trav_in_until()`:
      ! The graph contains no edges.
    Code
      trav_out(graph_no_edges)
    Condition
      Error in `trav_out()`:
      ! The graph contains no edges.
    Code
      trav_out_edge(graph_no_edges)
    Condition
      Error in `trav_out_edge()`:
      ! The graph contains no edges.
    Code
      trav_out_node(graph_no_edges)
    Condition
      Error in `trav_out_node()`:
      ! The graph contains no edges.
    Code
      trav_out_until(graph_no_edges)
    Condition
      Error in `trav_out_until()`:
      ! The graph contains no edges.
    Code
      trav_reverse_edge(graph_no_edges)
    Condition
      Error in `trav_reverse_edge()`:
      ! The graph contains no edges.

